@@ -37,7 +37,9 @@ import java.util.function.Predicate;
 import java.util.regex.Pattern;
 import java.util.stream.Collectors;
 
-import org.opennms.features.jest.client.SearchResultUtils;
+import org.opennms.features.elastic.client.ElasticRestClient;
+import org.opennms.features.elastic.client.model.SearchRequest;
+import org.opennms.features.elastic.client.model.SearchResponse;
 import org.opennms.features.jest.client.index.IndexSelector;
 import org.opennms.netmgt.flows.api.Conversation;
 import org.opennms.netmgt.flows.api.ConversationKey;
@@ -51,11 +53,11 @@ import org.opennms.netmgt.flows.processing.ConversationKeyUtils;
 
 import com.google.common.collect.ImmutableTable;
 import com.google.common.collect.Table;
+import com.google.gson.JsonArray;
+import com.google.gson.JsonElement;
 import com.google.gson.JsonObject;
 import com.google.gson.JsonPrimitive;
 
-import io.searchbox.client.JestClient;
-import io.searchbox.core.SearchResult;
 import io.searchbox.core.search.aggregation.MetricAggregation;
 import io.searchbox.core.search.aggregation.TermsAggregation;
 
@@ -66,14 +68,14 @@ public class RawFlowQueryService extends ElasticFlowQueryService {
 
     private final SearchQueryProvider searchQueryProvider = new SearchQueryProvider();
 
-    public RawFlowQueryService(JestClient client, IndexSelector indexSelector) {
+    public RawFlowQueryService(ElasticRestClient client, IndexSelector indexSelector) {
         super(client, indexSelector);
     }
 
     @Override
     public CompletableFuture<Long> getFlowCount(List<Filter> filters) {
         final String query = searchQueryProvider.getFlowCountQuery(filters);
-        return searchAsync(query, extractTimeRangeFilter(filters)).thenApply(SearchResultUtils::getTotal);
+        return searchAsync(query, extractTimeRangeFilter(filters)).thenApply(res -> res.getHits().getTotalHits());
     }
 
     @Override
@@ -220,7 +222,13 @@ public class RawFlowQueryService extends ElasticFlowQueryService {
         final TimeRangeFilter timeRangeFilter = extractTimeRangeFilter(filters);
         return searchAsync(searchQueryProvider.getAllValues(field.fieldName, field.size, filters), timeRangeFilter)
                 .thenApply(res -> {
-                    List<String> fieldValues = compositeKeyValuesPath(field.fieldName, GPath.string()).eval(res.getJsonObject());
+                    // Create a wrapper JSON object that contains aggregations
+                    JsonObject wrapper = new JsonObject();
+                    if (res.getAggregations() != null) {
+                        wrapper.add("aggregations", res.getAggregations());
+                        wrapper.add("aggs", res.getAggregations()); // Jest compatibility
+                    }
+                    List<String> fieldValues = compositeKeyValuesPath(field.fieldName, GPath.string()).eval(wrapper);
                     return fieldValues != null ? fieldValues : Collections.emptyList();
                 });
     }
@@ -248,7 +256,10 @@ public class RawFlowQueryService extends ElasticFlowQueryService {
         final String hostnameQuery = searchQueryProvider.getHostnameByConversationQuery(convoKey, filters);
         return searchAsync(hostnameQuery, timeRangeFilter)
                 .thenApply(res ->  {
-                    final JsonObject hit = res.getFirstHit(JsonObject.class).source;
+                    final JsonObject hit = AggregationUtils.getFirstHitSource(res);
+                    if (hit == null) {
+                        return result.build();
+                    }
                     if (Objects.equals(hit.getAsJsonPrimitive("netflow.src_addr").getAsString(), key.getLowerIp())) {
                         Optional.ofNullable(hit.getAsJsonPrimitive("netflow.src_addr_hostname")).map(JsonPrimitive::getAsString).ifPresent(result::withLowerHostname);
                         Optional.ofNullable(hit.getAsJsonPrimitive("netflow.dst_addr_hostname")).map(JsonPrimitive::getAsString).ifPresent(result::withUpperHostname);
@@ -274,7 +285,10 @@ public class RawFlowQueryService extends ElasticFlowQueryService {
         final String hostnameQuery = searchQueryProvider.getHostnameByHostQuery(host, filters);
         return searchAsync(hostnameQuery, timeRangeFilter)
                 .thenApply(res ->  {
-                    final JsonObject hit = res.getFirstHit(JsonObject.class).source;
+                    final JsonObject hit = AggregationUtils.getFirstHitSource(res);
+                    if (hit == null) {
+                        return result.build();
+                    }
                     if (Objects.equals(hit.getAsJsonPrimitive("netflow.src_addr").getAsString(), host)) {
                         Optional.ofNullable(hit.getAsJsonPrimitive("netflow.src_addr_hostname")).map(JsonPrimitive::getAsString).ifPresent(result::withHostname);
                     } else if (Objects.equals(hit.getAsJsonPrimitive("netflow.dst_addr").getAsString(), host)) {
@@ -377,13 +391,13 @@ public class RawFlowQueryService extends ElasticFlowQueryService {
         return seriesFuture.thenApply(builder -> TableUtils.sortTableByRowKeys(builder.build(), topN));
     }
 
-    private static ImmutableTable.Builder<Directional<String>, Long, Double> toTable(ImmutableTable.Builder<Directional<String>, Long, Double> builder, SearchResult res) {
-        final MetricAggregation aggs = res.getAggregations();
+    private static ImmutableTable.Builder<Directional<String>, Long, Double> toTable(ImmutableTable.Builder<Directional<String>, Long, Double> builder, SearchResponse res) {
+        final JsonObject aggs = res.getAggregations();
         if (aggs == null) {
             // No results
             return builder;
         }
-        final TermsAggregation groupedBy = aggs.getTermsAggregation("grouped_by");
+        final TermsAggregation groupedBy = AggregationUtils.getTermsAggregation(aggs, "grouped_by");
         if (groupedBy == null) {
             // No results
             return builder;
@@ -452,12 +466,12 @@ public class RawFlowQueryService extends ElasticFlowQueryService {
             final String bytesFromOthersQuery = searchQueryProvider.getSeriesFromOthersQuery(from, step, start, end,
                     groupByTerm, missingTermIncluded, filters);
             summariesFuture = summariesFuture.thenCombine(searchAsync(bytesFromOthersQuery, timeRangeFilter), (summaries, results) -> {
-                final MetricAggregation aggs = results.getAggregations();
+                final JsonObject aggs = results.getAggregations();
                 if (aggs == null) {
                     // No results
                     return summaries;
                 }
-                final TermsAggregation directionAgg = aggs.getTermsAggregation("direction");
+                final TermsAggregation directionAgg = AggregationUtils.getTermsAggregation(aggs, "direction");
                 if (directionAgg == null) {
                     // No results
                     return summaries;
@@ -540,7 +554,7 @@ public class RawFlowQueryService extends ElasticFlowQueryService {
             // We also need to query for items with a missing term, this will require a separate query
             final String seriesFromMissingQuery =
                     searchQueryProvider.getSeriesFromMissingQuery(fag.step, fag.start, fag.end, fag.groupByTerm, keyForMissingTerm, fag.filters);
-            CompletableFuture<SearchResult> missingFuture = searchAsync(seriesFromMissingQuery, fag.timeRangeFilter);
+            CompletableFuture<SearchResponse> missingFuture = searchAsync(seriesFromMissingQuery, fag.timeRangeFilter);
             return builder -> missingFuture.thenApply(searchResult -> toTable(builder, searchResult));
         } else {
             return keepValue();
@@ -567,7 +581,7 @@ public class RawFlowQueryService extends ElasticFlowQueryService {
             // We also need to query for items with a missing term, this will require a separate query
             String bytesFromMissingQuery
                     = searchQueryProvider.getSeriesFromMissingQuery(fag.step, fag.start, fag.end, fag.groupByTerm, keyForMissingTerm, fag.filters);
-            CompletableFuture<SearchResult> missingFuture = searchAsync(bytesFromMissingQuery, fag.timeRangeFilter);
+            CompletableFuture<SearchResponse> missingFuture = searchAsync(bytesFromMissingQuery, fag.timeRangeFilter);
             return summaries -> missingFuture.thenApply(results -> {
                 summaries.putAll(toTrafficSummaries(results));
                 return summaries;
@@ -596,15 +610,15 @@ public class RawFlowQueryService extends ElasticFlowQueryService {
                d1 ? -1 : d2 ? 1 : s1.getEntity().compareTo(s2.getEntity());
     };
 
-    private static Map<String, TrafficSummary<String>> toTrafficSummaries(SearchResult res) {
+    private static Map<String, TrafficSummary<String>> toTrafficSummaries(SearchResponse res) {
         // Build the traffic summaries from the search results
         final Map<String, TrafficSummary<String>> summaries = new LinkedHashMap<>();
-        final MetricAggregation aggs = res.getAggregations();
+        final JsonObject aggs = res.getAggregations();
         if (aggs == null) {
             // No results
             return summaries;
         }
-        final TermsAggregation groupedBy = aggs.getTermsAggregation("grouped_by");
+        final TermsAggregation groupedBy = AggregationUtils.getTermsAggregation(aggs, "grouped_by");
         if (groupedBy == null) {
             // No results
             return summaries;
@@ -638,14 +652,14 @@ public class RawFlowQueryService extends ElasticFlowQueryService {
                 .thenCompose((topN) -> getTotalBytesFrom(topN, groupByTerm, keyForMissingTerm, includeOther, filters));
     }
 
-    private static ImmutableTable.Builder<Directional<String>, Long, Double> processOthersResult(SearchResult res,
+    private static ImmutableTable.Builder<Directional<String>, Long, Double> processOthersResult(SearchResponse res,
                                                                                                  ImmutableTable.Builder<Directional<String>, Long, Double> builder) {
-        final MetricAggregation aggs = res.getAggregations();
+        final JsonObject aggs = res.getAggregations();
         if (aggs == null) {
             // No results
             return builder;
         }
-        final TermsAggregation directionAgg = aggs.getTermsAggregation("direction");
+        final TermsAggregation directionAgg = AggregationUtils.getTermsAggregation(aggs, "direction");
         if (directionAgg == null) {
             // No results
             return builder;
@@ -662,13 +676,13 @@ public class RawFlowQueryService extends ElasticFlowQueryService {
         return builder;
     }
 
-    private static List<String> processGroupedByResult(SearchResult searchResult, long limit) {
-        final MetricAggregation aggs = searchResult.getAggregations();
+    private static List<String> processGroupedByResult(SearchResponse searchResult, long limit) {
+        final JsonObject aggs = searchResult.getAggregations();
         if (aggs == null) {
             // No results
             return Collections.emptyList();
         }
-        final TermsAggregation groupedBy = aggs.getTermsAggregation("grouped_by");
+        final TermsAggregation groupedBy = AggregationUtils.getTermsAggregation(aggs, "grouped_by");
         if (groupedBy == null) {
             // No results
             return Collections.emptyList();
